@@ -30,11 +30,59 @@ FOR EACH ROW BEGIN
     SELECT RAISE(ABORT, 'corrupt definition of authority_list')
         WHERE (SELECT 1 FROM authority_list LIMIT 1) = 0;
 
+    -- check that a usage is registered for most objects where this is needed
+    SELECT RAISE(ABORT, 'One or several objects lack a corresponding record in the usage table')
+        WHERE EXISTS (
+            SELECT * FROM object_view o WHERE NOT EXISTS (
+                SELECT 1 FROM usage u WHERE
+                    o.table_name = u.object_table_name AND
+                    o.auth_name = u.object_auth_name AND
+                    o.code = u.object_code)
+            AND o.table_name NOT IN ('unit_of_measure', 'axis',
+                'celestial_body', 'ellipsoid', 'prime_meridian', 'extent')
+            -- the IGNF registry lacks extent for the following objects
+            AND NOT (o.auth_name = 'IGNF' AND o.table_name IN ('geodetic_datum', 'vertical_datum', 'conversion'))
+        );
+
+    SELECT RAISE(ABORT, 'Geodetic datum ensemble defined, but no ensemble member')
+        WHERE EXISTS (
+            SELECT * FROM geodetic_datum d WHERE ensemble_accuracy IS NOT NULL
+            AND NOT EXISTS (SELECT 1 FROM geodetic_datum_ensemble_member WHERE
+                d.auth_name = ensemble_auth_name AND d.code = ensemble_code)
+        );
+
+    SELECT RAISE(ABORT, 'Vertical datum ensemble defined, but no ensemble member')
+        WHERE EXISTS (
+            SELECT * FROM vertical_datum d WHERE ensemble_accuracy IS NOT NULL
+            AND NOT EXISTS (SELECT 1 FROM vertical_datum_ensemble_member WHERE
+                d.auth_name = ensemble_auth_name AND d.code = ensemble_code)
+        );
+
+    SELECT RAISE(ABORT, 'PROJ defines an alias that exists in EPSG')
+        WHERE EXISTS (
+         SELECT * FROM (
+            SELECT count(*) AS count, table_name, auth_name, code, alt_name FROM alias_name
+            WHERE source in ('EPSG', 'PROJ')
+            AND NOT (source = 'PROJ' AND alt_name IN ('GGRS87', 'NAD27', 'NAD83'))
+            GROUP BY table_name, auth_name, code, alt_name) x WHERE count > 1
+        );
+
     -- test to check that our custom grid transformation overrides are really needed
     SELECT RAISE(ABORT, 'PROJ grid_transformation defined whereas EPSG has one')
-        WHERE EXISTS (SELECT 1 FROM grid_transformation g1, grid_transformation g2 WHERE
-            lower(g1.grid_name) = lower(g2.grid_name) AND
-            g1.auth_name = 'PROJ' AND g2.auth_name = 'EPSG');
+        WHERE EXISTS (SELECT 1 FROM grid_transformation g1
+                      JOIN grid_transformation g2
+                      ON g1.source_crs_auth_name = g2.source_crs_auth_name
+                      AND g1.source_crs_code = g2.source_crs_code
+                      AND g1.target_crs_auth_name = g2.target_crs_auth_name
+                      AND g1.target_crs_code = g2.target_crs_code
+                      WHERE g1.auth_name = 'PROJ' AND g1.code NOT LIKE '%_RESTRICTED_TO_VERTCRS%' AND g2.auth_name = 'EPSG')
+        OR EXISTS (SELECT 1 FROM grid_transformation g1
+                      JOIN grid_transformation g2
+                      ON g1.source_crs_auth_name = g2.target_crs_auth_name
+                      AND g1.source_crs_code = g2.target_crs_code
+                      AND g1.target_crs_auth_name = g1.source_crs_auth_name
+                      AND g1.target_crs_code = g1.source_crs_code
+                      WHERE g1.auth_name = 'PROJ' AND g1.code NOT LIKE '%_RESTRICTED_TO_VERTCRS%' AND g2.auth_name = 'EPSG');
 
     SELECT RAISE(ABORT, 'Arg! there is now a EPSG:102100 object. Hack in createFromUserInput() will no longer work')
         WHERE EXISTS(SELECT 1 FROM crs_view WHERE auth_name = 'EPSG' AND code = '102100');
@@ -63,19 +111,6 @@ FOR EACH ROW BEGIN
                       (SELECT auth_name || code FROM geodetic_crs
                        WHERE type = 'geographic 2D'));
 
-    -- check that grids with HEIGHT_TO_GEOGRAPHIC3D method are properly registered
-    SELECT RAISE(ABORT, 'One grid_transformation with HEIGHT_TO_GEOGRAPHIC3D has not its source_crs in vertical_crs table')
-        WHERE EXISTS (SELECT * FROM grid_transformation g WHERE
-                      g.method_code = 'HEIGHT_TO_GEOGRAPHIC3D' AND
-                      g.source_crs_auth_name || g.source_crs_code NOT IN
-                      (SELECT auth_name || code FROM vertical_crs));
-    SELECT RAISE(ABORT, 'One grid_transformation with HEIGHT_TO_GEOGRAPHIC3D has not its target_crs in geodetic_crs table with type = ''geographic 3D''')
-        WHERE EXISTS (SELECT * FROM grid_transformation g WHERE
-                      g.method_code = 'HEIGHT_TO_GEOGRAPHIC3D' AND
-                      g.target_crs_auth_name || g.target_crs_code NOT IN
-                      (SELECT auth_name || code FROM geodetic_crs
-                       WHERE type = 'geographic 3D'));
-
     -- check that grids with Geographic3D to GravityRelatedHeight method are properly registered
     SELECT RAISE(ABORT, 'One grid_transformation with Geographic3D to GravityRelatedHeight has not its target_crs in vertical_crs table')
         WHERE EXISTS (SELECT * FROM grid_transformation g WHERE
@@ -95,27 +130,39 @@ FOR EACH ROW BEGIN
     -- check that transformations intersect the area of use of their source/target CRS
     -- EPSG, ESRI and IGNF have cases where this does not hold.
     SELECT RAISE(ABORT, 'The area of use of at least one coordinate_operation does not intersect the one of its source CRS')
-        WHERE EXISTS (SELECT * FROM coordinate_operation_view v, crs_view c, area va, area ca WHERE
+        WHERE EXISTS (SELECT * FROM coordinate_operation_view v, crs_view c, usage vu, extent ve, usage cu, extent ce WHERE
                       v.deprecated = 0 AND
                       v.auth_name NOT IN ('EPSG', 'ESRI', 'IGNF') AND
                       v.source_crs_auth_name = c.auth_name AND
                       v.source_crs_code = c.code AND
-                      v.area_of_use_auth_name = va.auth_name AND
-                      v.area_of_use_code = va.code AND
-                      c.area_of_use_auth_name = ca.auth_name AND
-                      c.area_of_use_code = ca.code AND
-                      NOT (ca.south_lat < va.north_lat AND va.south_lat < ca.north_lat));
+                      vu.object_table_name = v.table_name AND
+                      vu.object_auth_name = v.auth_name AND
+                      vu.object_code = v.code AND
+                      vu.extent_auth_name = ve.auth_name AND
+                      vu.extent_code = ve.code AND
+                      cu.object_table_name = c.table_name AND
+                      cu.object_auth_name = c.auth_name AND
+                      cu.object_code = c.code AND
+                      cu.extent_auth_name = ce.auth_name AND
+                      cu.extent_code = ce.code AND
+                      NOT (ce.south_lat < ve.north_lat AND ve.south_lat < ce.north_lat));
     SELECT RAISE(ABORT, 'The area of use of at least one coordinate_operation does not intersect the one of its target CRS')
-        WHERE EXISTS (SELECT * FROM coordinate_operation_view v, crs_view c, area va, area ca WHERE
+        WHERE EXISTS (SELECT * FROM coordinate_operation_view v, crs_view c, usage vu, extent ve, usage cu, extent ce WHERE
                       v.deprecated = 0 AND
                       v.auth_name NOT IN ('EPSG', 'ESRI', 'IGNF') AND
                       v.target_crs_auth_name = c.auth_name AND
                       v.target_crs_code = c.code AND
-                      v.area_of_use_auth_name = va.auth_name AND
-                      v.area_of_use_code = va.code AND
-                      c.area_of_use_auth_name = ca.auth_name AND
-                      c.area_of_use_code = ca.code AND
-                      NOT (ca.south_lat < va.north_lat AND va.south_lat < ca.north_lat));
+                      vu.object_table_name = v.table_name AND
+                      vu.object_auth_name = v.auth_name AND
+                      vu.object_code = v.code AND
+                      vu.extent_auth_name = ve.auth_name AND
+                      vu.extent_code = ve.code AND
+                      cu.object_table_name = c.table_name AND
+                      cu.object_auth_name = c.auth_name AND
+                      cu.object_code = c.code AND
+                      cu.extent_auth_name = ce.auth_name AND
+                      cu.extent_code = ce.code AND
+                      NOT (ce.south_lat < ve.north_lat AND ve.south_lat < ce.north_lat));
 
     -- check geoid_model table
     SELECT RAISE(ABORT, 'missing GEOID99 in geoid_model')
@@ -132,6 +179,12 @@ FOR EACH ROW BEGIN
         WHERE NOT EXISTS(SELECT 1 FROM geoid_model WHERE name = 'GEOID12B');
     SELECT RAISE(ABORT, 'missing GEOID18 in geoid_model')
         WHERE NOT EXISTS(SELECT 1 FROM geoid_model WHERE name = 'GEOID18');
+
+    -- check presence of au_ga_AUSGeoid98.tif
+    SELECT RAISE(ABORT, 'missing au_ga_AUSGeoid98.tif')
+        WHERE NOT EXISTS(SELECT 1 FROM grid_alternatives WHERE proj_grid_name = 'au_ga_AUSGeoid98.tif');
+
+
 END;
 INSERT INTO dummy DEFAULT VALUES;
 DROP TRIGGER final_checks;
