@@ -33,10 +33,6 @@
 #error "proj_internal.h can only be included from a C++ file"
 #endif
 
-#ifndef ACCEPT_USE_OF_DEPRECATED_PROJ_API_H
-#define ACCEPT_USE_OF_DEPRECATED_PROJ_API_H
-#endif
-
 #ifdef _MSC_VER
 #  ifndef _CRT_SECURE_NO_DEPRECATE
 #    define _CRT_SECURE_NO_DEPRECATE
@@ -66,20 +62,12 @@
 
 #include "proj.h"
 
-#ifdef PROJ_API_H
-#error proj_internal.h must be included before proj_api.h
-#endif
-
 #ifdef PROJ_RENAME_SYMBOLS
 #include "proj_symbol_rename.h"
 #endif
 
 #define STATIC_ASSERT(COND) ((void)sizeof(char[(COND) ? 1 : -1]))
 
-#if !defined(HAVE_C99_MATH)
-#define HAVE_C99_MATH 0
-#endif
-
 #ifndef PJ_TODEG
 #define PJ_TODEG(rad)  ((rad)*180.0/M_PI)
 #endif
@@ -178,7 +166,9 @@ enum pj_io_units {
     PJ_IO_UNITS_CLASSIC   = 1,  /* Scaled meters (right), projected system */
     PJ_IO_UNITS_PROJECTED = 2,  /* Meters, projected system */
     PJ_IO_UNITS_CARTESIAN = 3,  /* Meters, 3D cartesian system */
-    PJ_IO_UNITS_RADIANS   = 4   /* Radians */
+    PJ_IO_UNITS_RADIANS   = 4,  /* Radians */
+    PJ_IO_UNITS_DEGREES   = 5,  /* Degrees */
+
 };
 enum pj_io_units pj_left (PJ *P);
 enum pj_io_units pj_right (PJ *P);
@@ -199,15 +189,10 @@ PJ_COORD pj_inv4d (PJ_COORD coo, PJ *P);
 PJ_COORD PROJ_DLL pj_approx_2D_trans (PJ *P, PJ_DIRECTION direction, PJ_COORD coo);
 PJ_COORD PROJ_DLL pj_approx_3D_trans (PJ *P, PJ_DIRECTION direction, PJ_COORD coo);
 
+/* Provision for gettext translatable strings */
+#define _(str) (str)
 
-/* Grid functionality */
-int             proj_vgrid_init(PJ *P, const char *grids);
-int             proj_hgrid_init(PJ *P, const char *grids);
-double          proj_vgrid_value(PJ *P, PJ_LP lp, double vmultiplier);
-PJ_LP           proj_hgrid_value(PJ *P, PJ_LP lp);
-PJ_LP           proj_hgrid_apply(PJ *P, PJ_LP lp, PJ_DIRECTION direction);
-
-void PROJ_DLL proj_log_error (PJ *P, const char *fmt, ...);
+void PROJ_DLL proj_log_error (const PJ *P, const char *fmt, ...);
 void proj_log_debug (PJ *P, const char *fmt, ...);
 void proj_log_trace (PJ *P, const char *fmt, ...);
 
@@ -226,9 +211,6 @@ size_t pj_trim_argc (char *args);
 char **pj_trim_argv (size_t argc, char *args);
 char  *pj_make_args (size_t argc, char **argv);
 
-/* Lowest level: Minimum support for fileapi */
-void proj_fileapi_set (PJ *P, void *fileapi);
-
 typedef struct { double r, i; } COMPLEX;
 
 /* Forward declarations and typedefs for stuff needed inside the PJ object */
@@ -261,8 +243,6 @@ struct PJ_AREA {
     double  north_lat_degree;
 };
 
-struct projCtx_t;
-typedef struct projCtx_t projCtx_t;
 
 /*****************************************************************************
 
@@ -302,6 +282,62 @@ typedef    PJ_COORD  (* PJ_OPERATOR)    (PJ_COORD, PJ *);
 #define PJD_GRIDSHIFT 3
 #define PJD_WGS84     4   /* WGS84 (or anything considered equivalent) */
 
+struct CoordOperation
+{
+    int idxInOriginalList;
+    double minxSrc = 0.0;
+    double minySrc = 0.0;
+    double maxxSrc = 0.0;
+    double maxySrc = 0.0;
+    double minxDst = 0.0;
+    double minyDst = 0.0;
+    double maxxDst = 0.0;
+    double maxyDst = 0.0;
+    PJ* pj = nullptr;
+    std::string name{};
+    double accuracy = -1.0;
+    bool isOffshore = false;
+
+    CoordOperation(int idxInOriginalListIn,
+                   double minxSrcIn, double minySrcIn, double maxxSrcIn, double maxySrcIn,
+                   double minxDstIn, double minyDstIn, double maxxDstIn, double maxyDstIn,
+                   PJ* pjIn, const std::string& nameIn, double accuracyIn, bool isOffshoreIn):
+        idxInOriginalList(idxInOriginalListIn),
+        minxSrc(minxSrcIn), minySrc(minySrcIn), maxxSrc(maxxSrcIn), maxySrc(maxySrcIn),
+        minxDst(minxDstIn), minyDst(minyDstIn), maxxDst(maxxDstIn), maxyDst(maxyDstIn),
+        pj(pjIn), name(nameIn),
+        accuracy(accuracyIn),
+        isOffshore(isOffshoreIn)
+    {
+    }
+
+    CoordOperation(const CoordOperation&) = delete;
+
+    CoordOperation(CoordOperation&& other):
+        idxInOriginalList(other.idxInOriginalList),
+        minxSrc(other.minxSrc), minySrc(other.minySrc), maxxSrc(other.maxxSrc), maxySrc(other.maxySrc),
+        minxDst(other.minxDst), minyDst(other.minyDst), maxxDst(other.maxxDst), maxyDst(other.maxyDst),
+        name(std::move(other.name)),
+        accuracy(other.accuracy),
+        isOffshore(other.isOffshore) {
+        pj = other.pj;
+        other.pj = nullptr;
+    }
+
+    CoordOperation& operator=(const CoordOperation&) = delete;
+
+    ~CoordOperation()
+    {
+        proj_destroy(pj);
+    }
+};
+
+enum class TMercAlgo
+{
+    AUTO, // Poder/Engsager if far from central meridian, otherwise Evenden/Snyder
+    EVENDEN_SNYDER,
+    PODER_ENGSAGER,
+};
 
 /* base projection data structure */
 struct PJconsts {
@@ -317,7 +353,8 @@ struct PJconsts {
 
     **************************************************************************************/
 
-    projCtx_t *ctx = nullptr;
+    PJ_CONTEXT *ctx = nullptr;
+    const char *short_name = nullptr; /* From pj_list.h */
     const char *descr = nullptr;   /* From pj_list.h or individual PJ_*.c file */
     paralist *params = nullptr;    /* Parameter list */
     char *def_full = nullptr;      /* Full textual definition (usually 0 - set by proj_pj_info) */
@@ -358,6 +395,7 @@ struct PJconsts {
     PJ_OPERATOR inv4d = nullptr;
 
     PJ_DESTRUCTOR destructor = nullptr;
+    void   (*reassign_context)(PJ*, PJ_CONTEXT*) = nullptr;
 
 
     /*************************************************************************************
@@ -425,7 +463,6 @@ struct PJconsts {
     int  geoc = 0;                  /* Geocentric latitude flag */
     int  is_latlong = 0;            /* proj=latlong ... not really a projection at all */
     int  is_geocent = 0;            /* proj=geocent ... not really a projection at all */
-    int  is_pipeline = 0;           /* 1 if PJ represents a pipeline */
     int  need_ellps = 0;            /* 0 for operations that are purely cartesian */
     int  skip_fwd_prepare = 0;
     int  skip_fwd_finalize = 0;
@@ -482,32 +519,16 @@ struct PJconsts {
 
     int     datum_type = PJD_UNKNOWN;  /* PJD_UNKNOWN/3PARAM/7PARAM/GRIDSHIFT/WGS84 */
     double  datum_params[7] = {0,0,0,0,0,0,0}; /* Parameters for 3PARAM and 7PARAM */
-    struct _pj_gi **gridlist = nullptr;     /* TODO: Description needed */
-    int     gridlist_count = 0;
 
-    int     has_geoid_vgrids = 0;      /* TODO: Description needed */
-    struct _pj_gi **vgridlist_geoid = nullptr;   /* TODO: Description needed */
-    int     vgridlist_geoid_count = 0;
+    int     has_geoid_vgrids = 0;      /* used by legacy transform.cpp */
+    void*   hgrids_legacy = nullptr;   /* used by legacy transform.cpp. Is a pointer to a ListOfHGrids* */ 
+    void*   vgrids_legacy = nullptr;   /* used by legacy transform.cpp. Is a pointer to a ListOfVGrids* */ 
 
     double  from_greenwich = 0.0;       /* prime meridian offset (in radians) */
     double  long_wrap_center = 0.0;     /* 0.0 for -180 to 180, actually in radians*/
     int     is_long_wrap_set = 0;
     char    axis[4] = {0,0,0,0};        /* Axis order, pj_transform/pj_adjust_axis */
 
-    /* New Datum Shift Grid Catalogs */
-    char   *catalog_name = nullptr;
-    struct _PJ_GridCatalog *catalog = nullptr;
-
-    double  datum_date = 0.0;           /* TODO: Description needed */
-
-    struct _pj_gi *last_before_grid = nullptr;    /* TODO: Description needed */
-    PJ_Region     last_before_region = {0,0,0,0}; /* TODO: Description needed */
-    double        last_before_date = 0.0;         /* TODO: Description needed */
-
-    struct _pj_gi *last_after_grid = nullptr;     /* TODO: Description needed */
-    PJ_Region     last_after_region = {0,0,0,0};  /* TODO: Description needed */
-    double        last_after_date = 0.0;          /* TODO: Description needed */
-
     /*************************************************************************************
      ISO-19111 interface
     **************************************************************************************/
@@ -524,44 +545,6 @@ struct PJconsts {
     /*************************************************************************************
      proj_create_crs_to_crs() alternative coordinate operations
     **************************************************************************************/
-
-    struct CoordOperation
-    {
-        double minxSrc = 0.0;
-        double minySrc = 0.0;
-        double maxxSrc = 0.0;
-        double maxySrc = 0.0;
-        double minxDst = 0.0;
-        double minyDst = 0.0;
-        double maxxDst = 0.0;
-        double maxyDst = 0.0;
-        PJ* pj = nullptr;
-        std::string name{};
-
-        CoordOperation(double minxSrcIn, double minySrcIn, double maxxSrcIn, double maxySrcIn,
-                       double minxDstIn, double minyDstIn, double maxxDstIn, double maxyDstIn,
-                       PJ* pjIn, const std::string& nameIn):
-            minxSrc(minxSrcIn), minySrc(minySrcIn), maxxSrc(maxxSrcIn), maxySrc(maxySrcIn),
-            minxDst(minxDstIn), minyDst(minyDstIn), maxxDst(maxxDstIn), maxyDst(maxyDstIn),
-            pj(pjIn), name(nameIn) {}
-
-        CoordOperation(const CoordOperation&) = delete;
-
-        CoordOperation(CoordOperation&& other):
-            minxSrc(other.minxSrc), minySrc(other.minySrc), maxxSrc(other.maxxSrc), maxySrc(other.maxySrc),
-            minxDst(other.minxDst), minyDst(other.minyDst), maxxDst(other.maxxDst), maxyDst(other.maxyDst),
-            name(std::move(other.name)) {
-                pj = other.pj;
-                other.pj = nullptr;
-            }
-
-        CoordOperation& operator=(const CoordOperation&) = delete;
-
-        ~CoordOperation()
-        {
-            proj_destroy(pj);
-        }
-    };
     std::vector<CoordOperation> alternativeCoordinateOperations{};
     int iCurCoordOp = -1;
 
@@ -583,7 +566,7 @@ struct PJconsts {
 struct ARG_list {
     paralist *next;
     char used;
-#if defined(__GNUC__) && __GNUC__ >= 8
+#if (defined(__GNUC__) && __GNUC__ >= 8) || (defined(__clang__) && __clang_major__ >= 9)
     char param[]; /* variable-length member */
     /* Safer to use [] for gcc 8. See https://github.com/OSGeo/proj.4/pull/1087 */
     /* and https://gcc.gnu.org/bugzilla/show_bug.cgi?id=86914 */
@@ -619,86 +602,60 @@ struct FACTORS {
     int    code;           /* always 0 */
 };
 
-/* library errors */
-#define PJD_ERR_NO_ARGS                  -1
-#define PJD_ERR_NO_OPTION_IN_INIT_FILE   -2
-#define PJD_ERR_NO_COLON_IN_INIT_STRING  -3
-#define PJD_ERR_PROJ_NOT_NAMED           -4
-#define PJD_ERR_UNKNOWN_PROJECTION_ID    -5
-#define PJD_ERR_INVALID_ECCENTRICITY     -6
-#define PJD_ERR_UNKNOWN_UNIT_ID          -7
-#define PJD_ERR_INVALID_BOOLEAN_PARAM    -8
-#define PJD_ERR_UNKNOWN_ELLP_PARAM       -9
-#define PJD_ERR_REV_FLATTENING_IS_ZERO  -10
-#define PJD_ERR_REF_RAD_LARGER_THAN_90  -11
-#define PJD_ERR_ES_LESS_THAN_ZERO       -12
-#define PJD_ERR_MAJOR_AXIS_NOT_GIVEN    -13
-#define PJD_ERR_LAT_OR_LON_EXCEED_LIMIT -14
-#define PJD_ERR_INVALID_X_OR_Y          -15
-#define PJD_ERR_WRONG_FORMAT_DMS_VALUE  -16
-#define PJD_ERR_NON_CONV_INV_MERI_DIST  -17
-#define PJD_ERR_NON_CON_INV_PHI2        -18
-#define PJD_ERR_ACOS_ASIN_ARG_TOO_LARGE -19
-#define PJD_ERR_TOLERANCE_CONDITION     -20
-#define PJD_ERR_CONIC_LAT_EQUAL         -21
-#define PJD_ERR_LAT_LARGER_THAN_90      -22
-#define PJD_ERR_LAT1_IS_ZERO            -23
-#define PJD_ERR_LAT_TS_LARGER_THAN_90   -24
-#define PJD_ERR_CONTROL_POINT_NO_DIST   -25
-#define PJD_ERR_NO_ROTATION_PROJ        -26
-#define PJD_ERR_W_OR_M_ZERO_OR_LESS     -27
-#define PJD_ERR_LSAT_NOT_IN_RANGE       -28
-#define PJD_ERR_PATH_NOT_IN_RANGE       -29
-#define PJD_ERR_INVALID_H               -30
-#define PJD_ERR_K_LESS_THAN_ZERO        -31
-#define PJD_ERR_LAT_1_OR_2_ZERO_OR_90   -32
-#define PJD_ERR_LAT_0_OR_ALPHA_EQ_90    -33
-#define PJD_ERR_ELLIPSOID_USE_REQUIRED  -34
-#define PJD_ERR_INVALID_UTM_ZONE        -35
-/* -36 no longer used */
-#define PJD_ERR_FAILED_TO_FIND_PROJ     -37
-#define PJD_ERR_FAILED_TO_LOAD_GRID     -38
-#define PJD_ERR_INVALID_M_OR_N          -39
-#define PJD_ERR_N_OUT_OF_RANGE          -40
-#define PJD_ERR_LAT_1_2_UNSPECIFIED     -41
-#define PJD_ERR_ABS_LAT1_EQ_ABS_LAT2    -42
-#define PJD_ERR_LAT_0_HALF_PI_FROM_MEAN -43
-#define PJD_ERR_UNPARSEABLE_CS_DEF      -44
-#define PJD_ERR_GEOCENTRIC              -45
-#define PJD_ERR_UNKNOWN_PRIME_MERIDIAN  -46
-#define PJD_ERR_AXIS                    -47
-#define PJD_ERR_GRID_AREA               -48
-#define PJD_ERR_INVALID_SWEEP_AXIS      -49
-#define PJD_ERR_MALFORMED_PIPELINE      -50
-#define PJD_ERR_UNIT_FACTOR_LESS_THAN_0 -51
-#define PJD_ERR_INVALID_SCALE           -52
-#define PJD_ERR_NON_CONVERGENT          -53
-#define PJD_ERR_MISSING_ARGS            -54
-#define PJD_ERR_LAT_0_IS_ZERO           -55
-#define PJD_ERR_ELLIPSOIDAL_UNSUPPORTED -56
-#define PJD_ERR_TOO_MANY_INITS          -57
-#define PJD_ERR_INVALID_ARG             -58
-#define PJD_ERR_INCONSISTENT_UNIT       -59
-#define PJD_ERR_MUTUALLY_EXCLUSIVE_ARGS -60
-#define PJD_ERR_GENERIC_ERROR           -61
-/* NOTE: Remember to update src/strerrno.cpp, src/apps/gie.cpp and transient_error in */
-/* src/transform.cpp when adding new value */
-
+// Legacy
 struct projFileAPI_t;
 
 struct projCppContext;
 
+struct projNetworkCallbacksAndData
+{
+    bool enabled = false;
+    bool enabled_env_variable_checked = false; // whereas we have checked PROJ_NETWORK env variable
+    proj_network_open_cbk_type open = nullptr;
+    proj_network_close_cbk_type close = nullptr;
+    proj_network_get_header_value_cbk_type get_header_value = nullptr;
+    proj_network_read_range_type read_range = nullptr;
+    void* user_data = nullptr;
+};
+
+struct projGridChunkCache
+{
+    bool enabled = true;
+    std::string filename{};
+    long long max_size = 300 * 1024 * 1024;
+    int ttl = 86400; // 1 day
+};
+
+struct projFileApiCallbackAndData
+{
+    PROJ_FILE_HANDLE* (*open_cbk)(PJ_CONTEXT *ctx, const char *filename, PROJ_OPEN_ACCESS access, void* user_data) = nullptr;
+    size_t           (*read_cbk)(PJ_CONTEXT *ctx, PROJ_FILE_HANDLE*, void* buffer, size_t size, void* user_data) = nullptr;
+    size_t           (*write_cbk)(PJ_CONTEXT *ctx, PROJ_FILE_HANDLE*, const void* buffer, size_t size, void* user_data) = nullptr;
+    int              (*seek_cbk)(PJ_CONTEXT *ctx, PROJ_FILE_HANDLE*, long long offset, int whence, void* user_data) = nullptr;
+    unsigned long long (*tell_cbk)(PJ_CONTEXT *ctx, PROJ_FILE_HANDLE*, void* user_data) = nullptr;
+    void             (*close_cbk)(PJ_CONTEXT *ctx, PROJ_FILE_HANDLE*, void* user_data) = nullptr;
+
+    int (*exists_cbk)(PJ_CONTEXT *ctx, const char *filename, void* user_data) = nullptr;
+    int (*mkdir_cbk)(PJ_CONTEXT *ctx, const char *filename, void* user_data) = nullptr;
+    int (*unlink_cbk)(PJ_CONTEXT *ctx, const char *filename, void* user_data) = nullptr;
+    int (*rename_cbk)(PJ_CONTEXT *ctx, const char *oldPath, const char *newPath, void* user_data) = nullptr;
+
+    void*            user_data = nullptr;
+};
+
 /* proj thread context */
-struct projCtx_t {
+struct pj_ctx{
+    std::string lastFullErrorMessage{}; // used by proj_context_errno_string
     int     last_errno = 0;
-    int     debug_level = 0;
+    int     debug_level = PJ_LOG_ERROR;
     void    (*logger)(void *, int, const char *) = nullptr;
     void    *logger_app_data = nullptr;
-    struct projFileAPI_t *fileapi = nullptr;
     struct projCppContext* cpp_context = nullptr; /* internal context for C++ code */
     int     use_proj4_init_rules = -1; /* -1 = unknown, 0 = no, 1 = yes */
     int     epsg_file_exists = -1; /* -1 = unknown, 0 = no, 1 = yes */
+    std::string ca_bundle_path{};
 
+    std::string env_var_proj_lib{}; // content of PROJ_LIB environment variable. Use Filemanager::getProjLibEnvVar() to access
     std::vector<std::string> search_paths{};
     const char **c_compat_paths = nullptr; // same, but for projinfo usage
 
@@ -706,17 +663,36 @@ struct projCtx_t {
     const char* (*file_finder) (PJ_CONTEXT *, const char*, void* user_data) = nullptr;
     void* file_finder_user_data = nullptr;
 
+    bool defer_grid_opening = false; // set transiently by pj_obj_create()
+
+    projFileApiCallbackAndData fileApi{};
+    std::string custom_sqlite3_vfs_name{};
+    std::string user_writable_directory{};
+
+    // BEGIN ini file settings
+    bool iniFileLoaded = false;
+    std::string endpoint{};
+    projNetworkCallbacksAndData networking{};
+    projGridChunkCache gridChunkCache{};
+    TMercAlgo defaultTmercAlgo = TMercAlgo::PODER_ENGSAGER; // can be overridden by content of proj.ini
+    // END ini file settings
+
     int projStringParserCreateFromPROJStringRecursionCounter = 0; // to avoid potential infinite recursion in PROJStringParser::createFromPROJString()
+    int pipelineInitRecursiongCounter = 0; // to avoid potential infinite recursion in pipeline.cpp
 
-    projCtx_t() = default;
-    projCtx_t(const projCtx_t&);
-    ~projCtx_t();
 
-    projCtx_t& operator= (const projCtx_t&) = delete;
+    pj_ctx() = default;
+    pj_ctx(const pj_ctx&);
+    ~pj_ctx();
 
+    pj_ctx& operator= (const pj_ctx&) = delete;
+
+    projCppContext* get_cpp_context();
+    void safeAutoCloseDbIfNeeded();
     void set_search_paths(const std::vector<std::string>& search_paths_in);
+    void set_ca_bundle_path(const std::string& ca_bundle_path_in);
 
-    static projCtx_t createDefault();
+    static pj_ctx createDefault();
 };
 
 /* Generate pj_list external or make list from include file */
@@ -731,7 +707,7 @@ C_NAMESPACE_VAR struct PJ_DATUMS pj_datums[];
 #ifdef PJ_LIB__
 #define PROJ_HEAD(name, desc) static const char des_##name [] = desc
 
-#define OPERATION(name, NEED_ELPJ_LPS)                          \
+#define OPERATION(name, NEED_ELLPS)                          \
                                                              \
 pj_projection_specific_setup_##name (PJ *P);                 \
 C_NAMESPACE PJ *pj_##name (PJ *P);                           \
@@ -744,8 +720,9 @@ C_NAMESPACE PJ *pj_##name (PJ *P) {                          \
     P = pj_new();                                            \
     if (nullptr==P)                                          \
         return nullptr;                                      \
+    P->short_name = #name;                                   \
     P->descr = des_##name;                                   \
-    P->need_ellps = NEED_ELPJ_LPS;                              \
+    P->need_ellps = NEED_ELLPS;                              \
     P->left  = PJ_IO_UNITS_RADIANS;                          \
     P->right = PJ_IO_UNITS_CLASSIC;                          \
     return P;                                                \
@@ -762,92 +739,43 @@ PJ *pj_projection_specific_setup_##name (PJ *P)
 
 #endif /* def PJ_LIB__ */
 
-
-#define MAX_TAB_ID 80
-typedef struct { float lam, phi; } FLP;
-typedef struct { pj_int32 lam, phi; } ILP;
-
-struct CTABLE {
-    char id[MAX_TAB_ID];    /* ascii info */
-    PJ_LP ll;               /* lower left corner coordinates */
-    PJ_LP del;              /* size of cells */
-    ILP lim;                /* limits of conversion matrix */
-    FLP *cvs;               /* conversion matrix */
-};
-
-typedef struct _pj_gi {
-    char *gridname;     /* identifying name of grid, eg "conus" or ntv2_0.gsb */
-    char *filename;     /* full path to filename */
-
-    const char *format; /* format of this grid, ie "ctable", "ntv1",
-                           "ntv2" or "missing". */
-
-    long   grid_offset;  /* offset in file, for delayed loading */
-    int   must_swap;    /* only for NTv2 */
-
-    struct CTABLE *ct;
-
-    struct _pj_gi *next;
-    struct _pj_gi *child;
-} PJ_GRIDINFO;
-
-typedef struct {
-    PJ_Region region;
-    int  priority;      /* higher used before lower */
-    double date;        /* year.fraction */
-    char *definition;   /* usually the gridname */
-
-    PJ_GRIDINFO  *gridinfo;
-    int available;      /* 0=unknown, 1=true, -1=false */
-} PJ_GridCatalogEntry;
-
-typedef struct _PJ_GridCatalog {
-    char *catalog_name;
-
-    PJ_Region region;   /* maximum extent of catalog data */
-
-    int entry_count;
-    PJ_GridCatalogEntry *entries;
-
-    struct _PJ_GridCatalog *next;
-} PJ_GridCatalog;
-
 /* procedure prototypes */
 double PROJ_DLL dmstor(const char *, char **);
-double dmstor_ctx(projCtx_t *ctx, const char *, char **);
+double dmstor_ctx(PJ_CONTEXT *ctx, const char *, char **);
 void   PROJ_DLL set_rtodms(int, int);
 char  PROJ_DLL *rtodms(char *, double, int, int);
 double PROJ_DLL adjlon(double);
-double aacos(projCtx_t *,double);
-double aasin(projCtx_t *,double);
+double aacos(PJ_CONTEXT *,double);
+double aasin(PJ_CONTEXT *,double);
 double asqrt(double);
 double aatan2(double, double);
 
-PROJVALUE PROJ_DLL pj_param(projCtx_t *ctx, paralist *, const char *);
+PROJVALUE PROJ_DLL pj_param(PJ_CONTEXT *ctx, paralist *, const char *);
 paralist PROJ_DLL *pj_param_exists (paralist *list, const char *parameter);
 paralist PROJ_DLL *pj_mkparam(const char *);
 paralist *pj_mkparam_ws (const char *str, const char **next_str);
 
 
-int PROJ_DLL pj_ell_set(projCtx_t *ctx, paralist *, double *, double *);
-int pj_datum_set(projCtx_t *,paralist *, PJ *);
+int PROJ_DLL pj_ell_set(PJ_CONTEXT *ctx, paralist *, double *, double *);
+int pj_datum_set(PJ_CONTEXT *,paralist *, PJ *);
 int pj_angular_units_set(paralist *, PJ *);
 
 paralist *pj_clone_paralist( const paralist* );
 paralist *pj_search_initcache( const char *filekey );
 void      pj_insert_initcache( const char *filekey, const paralist *list);
-paralist *pj_expand_init(projCtx_t *ctx, paralist *init);
+paralist *pj_expand_init(PJ_CONTEXT *ctx, paralist *init);
 
-void     *pj_dealloc_params (projCtx_t *ctx, paralist *start, int errlev);
+void     *free_params (PJ_CONTEXT *ctx, paralist *start, int errlev);
 
 
 double *pj_enfn(double);
-double  pj_mlfn(double, double, double, double *);
-double  pj_inv_mlfn(projCtx_t *, double, double, double *);
+double  pj_mlfn(double, double, double, const double *);
+double  pj_inv_mlfn(PJ_CONTEXT *, double, double, const double *);
 double  pj_qsfn(double, double, double);
 double  pj_tsfn(double, double, double);
 double  pj_msfn(double, double, double);
-double  PROJ_DLL pj_phi2(projCtx_t *, double, double);
+double  PROJ_DLL pj_phi2(PJ_CONTEXT *, const double, const double);
+double  pj_sinhpsi2tanphi(PJ_CONTEXT *, const double, const double);
 double  pj_qsfn_(double, PJ *);
 double *pj_authset(double);
 double  pj_authlat(double, double *);
@@ -858,56 +786,12 @@ COMPLEX pj_zpolyd1(COMPLEX, const COMPLEX *, int, COMPLEX *);
 int pj_deriv(PJ_LP, double, const PJ *, struct DERIVS *);
 int pj_factors(PJ_LP, const PJ *, double, struct FACTORS *);
 
-/* nadcon related protos */
-PJ_LP             nad_intr(PJ_LP, struct CTABLE *);
-PJ_LP             nad_cvt(PJ_LP, int, struct CTABLE *);
-struct CTABLE *nad_init(projCtx_t *ctx, char *);
-struct CTABLE *nad_ctable_init( projCtx_t *ctx, struct projFileAPI_t* fid );
-int            nad_ctable_load( projCtx_t *ctx, struct CTABLE *, struct projFileAPI_t* fid );
-struct CTABLE *nad_ctable2_init( projCtx_t *ctx, struct projFileAPI_t* fid );
-int            nad_ctable2_load( projCtx_t *ctx, struct CTABLE *, struct projFileAPI_t* fid );
-void           nad_free(struct CTABLE *);
-
-/* higher level handling of datum grid shift files */
-
-int pj_apply_vgridshift( PJ *defn, const char *listname,
-                         PJ_GRIDINFO ***gridlist_p,
-                         int *gridlist_count_p,
-                         int inverse,
-                         long point_count, int point_offset,
-                         double *x, double *y, double *z );
-int pj_apply_gridshift_2( PJ *defn, int inverse,
-                          long point_count, int point_offset,
-                          double *x, double *y, double *z );
-int pj_apply_gridshift_3( projCtx_t *ctx,
-                          PJ_GRIDINFO **gridlist, int gridlist_count,
-                          int inverse, long point_count, int point_offset,
-                          double *x, double *y, double *z );
-
-PJ_GRIDINFO **pj_gridlist_from_nadgrids( projCtx_t *, const char *, int * );
-
-PJ_GRIDINFO *pj_gridinfo_init( projCtx_t *, const char * );
-int          pj_gridinfo_load( projCtx_t *, PJ_GRIDINFO * );
-void         pj_gridinfo_free( projCtx_t *, PJ_GRIDINFO * );
-
-PJ_GridCatalog *pj_gc_findcatalog( projCtx_t *, const char * );
-PJ_GridCatalog *pj_gc_readcatalog( projCtx_t *, const char * );
-void pj_gc_unloadall( projCtx_t *);
-int pj_gc_apply_gridshift( PJ *defn, int inverse,
-                           long point_count, int point_offset,
-                           double *x, double *y, double *z );
-int pj_gc_apply_gridshift( PJ *defn, int inverse,
-                           long point_count, int point_offset,
-                           double *x, double *y, double *z );
-
-double pj_gc_parsedate( projCtx_t *, const char * );
-
 void  *proj_mdist_ini(double);
 double proj_mdist(double, double, double, const void *);
-double proj_inv_mdist(projCtx_t *ctx, double, const void *);
+double proj_inv_mdist(PJ_CONTEXT *ctx, double, const void *);
 void  *pj_gauss_ini(double, double, double *,double *);
-PJ_LP     pj_gauss(projCtx_t *, PJ_LP, const void *);
-PJ_LP     pj_inv_gauss(projCtx_t *, PJ_LP, const void *);
+PJ_LP     pj_gauss(PJ_CONTEXT *, PJ_LP, const void *);
+PJ_LP     pj_inv_gauss(PJ_CONTEXT *, PJ_LP, const void *);
 
 struct PJ_DATUMS           PROJ_DLL *pj_get_datums_ref( void );
 
@@ -918,7 +802,7 @@ double PROJ_DLL pj_atof( const char* nptr );
 double pj_strtod( const char *nptr, char **endptr );
 void   pj_freeup_plain (PJ *P);
 
-PJ* pj_init_ctx_with_allow_init_epsg( projCtx_t *ctx, int argc, char **argv, int allow_init_epsg );
+PJ* pj_init_ctx_with_allow_init_epsg( PJ_CONTEXT *ctx, int argc, char **argv, int allow_init_epsg );
 
 std::string PROJ_DLL pj_add_type_crs_if_needed(const std::string& str);
 std::string pj_double_quote_string_param_if_needed(const std::string& str);
@@ -926,7 +810,88 @@ std::string pj_double_quote_string_param_if_needed(const std::string& str);
 PJ *pj_create_internal (PJ_CONTEXT *ctx, const char *definition);
 PJ *pj_create_argv_internal (PJ_CONTEXT *ctx, int argc, char **argv);
 
-/* classic public API */
-#include "proj_api.h"
+// For use by projinfo
+void pj_load_ini(PJ_CONTEXT* ctx);
+
+// Exported for testing purposes only
+std::string PROJ_DLL pj_context_get_grid_cache_filename(PJ_CONTEXT *ctx);
+
+// For use by projsync
+void PROJ_DLL pj_context_set_user_writable_directory(PJ_CONTEXT* ctx, const std::string& path);
+std::string PROJ_DLL pj_get_relative_share_proj(PJ_CONTEXT *ctx);
+
+std::vector<CoordOperation> pj_create_prepared_operations(PJ_CONTEXT *ctx,
+                                                     const PJ *source_crs,
+                                                     const PJ *target_crs,
+                                                     PJ_OBJ_LIST* op_list);
+
+int pj_get_suggested_operation(PJ_CONTEXT *ctx,
+                               const std::vector<CoordOperation>& opList,
+                               const int iExcluded[2],
+                               PJ_DIRECTION direction,
+                               PJ_COORD coord);
+
+const PJ_UNITS *pj_list_linear_units();
+const PJ_UNITS *pj_list_angular_units();
+
+void pj_clear_hgridshift_knowngrids_cache();
+void pj_clear_vgridshift_knowngrids_cache();
+
+PJ_LP pj_generic_inverse_2d(PJ_XY xy, PJ *P, PJ_LP lpInitial);
+
+
+
+
+/*****************************************************************************/
+/*                                                                           */
+/*                              proj_api.h                                   */
+/*                                                                           */
+/*    The rest of this header file includes what used to be "proj_api.h"     */
+/*                                                                           */
+/*****************************************************************************/
+
+/* pj_init() and similar functions can be used with a non-C locale */
+/* Can be detected too at runtime if the symbol pj_atof exists */
+#define PJ_LOCALE_SAFE 1
+
+#define RAD_TO_DEG    57.295779513082321
+#define DEG_TO_RAD   .017453292519943296
+
+
+
+
+extern char const PROJ_DLL pj_release[]; /* global release id string */
+
+/* procedure prototypes */
+
+PJ_CONTEXT PROJ_DLL *pj_get_default_ctx(void);
+PJ_CONTEXT *pj_get_ctx( PJ *);
+
+PJ_XY PROJ_DLL pj_fwd(PJ_LP, PJ *);
+PJ_LP PROJ_DLL pj_inv(PJ_XY, PJ *);
+
+PJ_XYZ pj_fwd3d(PJ_LPZ, PJ *);
+PJ_LPZ pj_inv3d(PJ_XYZ, PJ *);
+
+
+void pj_clear_initcache(void);
+void PROJ_DLL pj_pr_list(PJ *); /* used by proj.cpp */
+char *pj_get_def(PJ *, int);
+int pj_has_inverse(PJ *);
+
+
+char *pj_strdup(const char *str);
+const char PROJ_DLL *pj_get_release(void);
+void pj_acquire_lock(void);
+void pj_release_lock(void);
+void pj_cleanup_lock(void);
+
+void pj_log( PJ_CONTEXT * ctx, int level, const char *fmt, ... );
+void pj_stderr_logger( void *, int, const char * );
+
+int pj_find_file(PJ_CONTEXT * ctx, const char *short_filename,
+                 char* out_full_filename, size_t out_full_filename_size);
+
+
 
 #endif /* ndef PROJ_INTERNAL_H */
