@@ -1,6 +1,6 @@
 /******************************************************************************
  * Project: PROJ.4
- * Purpose: Implementation of the HEAPJ_LPix and rHEAPJ_LPix projections.
+ * Purpose: Implementation of the HEALPix and rHEALPix projections.
  *          For background see <http://code.scenzgrid.org/index.php/p/scenzgrid-py/source/tree/master/docs/rhealpix_dggs.pdf>.
  * Authors: Alex Raichev (raichev@cs.auckland.ac.nz)
  *          Michael Speth (spethm@landcareresearch.co.nz)
@@ -36,8 +36,8 @@
 #include "proj.h"
 #include "proj_internal.h"
 
-PROJ_HEAD(healpix, "HEAPJ_LPix") "\n\tSph&Ell";
-PROJ_HEAD(rhealpix, "rHEAPJ_LPix") "\n\tSph&Ell\n\tnorth_square= south_square=";
+PROJ_HEAD(healpix, "HEALPix") "\n\tSph&Ell\n\trot_xy=";
+PROJ_HEAD(rhealpix, "rHEALPix") "\n\tSph&Ell\n\tnorth_square= south_square=";
 
 /* Matrix for counterclockwise rotation by pi/2: */
 # define R1 {{ 0,-1},{ 1, 0}}
@@ -56,6 +56,7 @@ namespace { // anonymous namespace
 struct pj_opaque {
     int north_square;
     int south_square;
+    double rot_xy;
     double qp;
     double *apa;
 };
@@ -78,6 +79,12 @@ static double sign (double v) {
     return v > 0 ? 1 : (v < 0 ? -1 : 0);
 }
 
+static PJ_XY rotate(PJ_XY p, double angle) {
+    PJ_XY result;
+    result.x = p.x * cos(angle) - p.y * sin(angle);
+    result.y = p.y * cos(angle) + p.x * sin(angle);
+    return result;
+}
 
 /**
  * Return the index of the matrix in ROT.
@@ -152,10 +159,10 @@ static int pnpoly(int nvert, double vert[][2], double testx, double testy) {
 
 /**
  * Return 1 if (x, y) lies in (the interior or boundary of) the image of the
- * HEAPJ_LPix projection (in case proj=0) or in the image the rHEAPJ_LPix projection
+ * HEALPix projection (in case proj=0) or in the image the rHEALPix projection
  * (in case proj=1), and return 0 otherwise.
- * @param north_square the position of the north polar square (rHEAPJ_LPix only)
- * @param south_square the position of the south polar square (rHEAPJ_LPix only)
+ * @param north_square the position of the north polar square (rHEALPix only)
+ * @param south_square the position of the south polar square (rHEALPix only)
  **/
 static int in_image(double x, double y, int proj, int north_square,
                     int south_square) {
@@ -178,45 +185,31 @@ static int in_image(double x, double y, int proj, int north_square,
             {-M_FORTPI,   -M_HALFPI - EPS},
             {-M_HALFPI,   -M_FORTPI - EPS},
             {-3*M_FORTPI, -M_HALFPI - EPS},
-            {-M_PI - EPS, -M_FORTPI}
+            {-M_PI - EPS, -M_FORTPI},
+            {-M_PI - EPS,  M_FORTPI}
         };
         return pnpoly((int)sizeof(healpixVertsJit)/
                       sizeof(healpixVertsJit[0]), healpixVertsJit, x, y);
     } else {
         /**
-         * Assigning each element by index to avoid warnings such as
-         * 'initializer element is not computable at load time'.
-         * Before C99 this was not allowed and to keep as portable as
-         * possible we do it the C89 way here.
          * We need to assign the array this way because the input is
          * dynamic (north_square and south_square vars are unknown at
          * compile time).
          **/
-        double rhealpixVertsJit[12][2];
-        rhealpixVertsJit[0][0]  = -M_PI - EPS;
-        rhealpixVertsJit[0][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[1][0]  = -M_PI + north_square*M_HALFPI- EPS;
-        rhealpixVertsJit[1][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[2][0]  = -M_PI + north_square*M_HALFPI- EPS;
-        rhealpixVertsJit[2][1]  =  3*M_FORTPI + EPS;
-        rhealpixVertsJit[3][0]  = -M_PI + (north_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[3][1]  =  3*M_FORTPI + EPS;
-        rhealpixVertsJit[4][0]  = -M_PI + (north_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[4][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[5][0]  =  M_PI + EPS;
-        rhealpixVertsJit[5][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[6][0]  =  M_PI + EPS;
-        rhealpixVertsJit[6][1]  = -M_FORTPI - EPS;
-        rhealpixVertsJit[7][0]  = -M_PI + (south_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[7][1]  = -M_FORTPI - EPS;
-        rhealpixVertsJit[8][0]  = -M_PI + (south_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[8][1]  = -3*M_FORTPI - EPS;
-        rhealpixVertsJit[9][0]  = -M_PI + south_square*M_HALFPI - EPS;
-        rhealpixVertsJit[9][1]  = -3*M_FORTPI - EPS;
-        rhealpixVertsJit[10][0] = -M_PI + south_square*M_HALFPI - EPS;
-        rhealpixVertsJit[10][1] = -M_FORTPI - EPS;
-        rhealpixVertsJit[11][0] = -M_PI - EPS;
-        rhealpixVertsJit[11][1] = -M_FORTPI - EPS;
+        double rhealpixVertsJit[][2] = {
+            { -M_PI - EPS,                                  M_FORTPI + EPS },
+            { -M_PI + north_square*M_HALFPI- EPS,           M_FORTPI + EPS },
+            { -M_PI + north_square*M_HALFPI- EPS,           3*M_FORTPI + EPS },
+            { -M_PI + (north_square + 1.0)*M_HALFPI + EPS,  3*M_FORTPI + EPS },
+            { -M_PI + (north_square + 1.0)*M_HALFPI + EPS,  M_FORTPI + EPS},
+            {  M_PI + EPS,                                  M_FORTPI + EPS },
+            {  M_PI + EPS,                                  -M_FORTPI - EPS },
+            { -M_PI + (south_square + 1.0)*M_HALFPI + EPS,  -M_FORTPI - EPS },
+            { -M_PI + (south_square + 1.0)*M_HALFPI + EPS,  -3*M_FORTPI - EPS },
+            { -M_PI + south_square*M_HALFPI - EPS,          -3*M_FORTPI - EPS },
+            { -M_PI + south_square*M_HALFPI - EPS,          -M_FORTPI - EPS },
+            { -M_PI - EPS,                                  -M_FORTPI - EPS }
+        };
 
         return pnpoly((int)sizeof(rhealpixVertsJit)/
                       sizeof(rhealpixVertsJit[0]), rhealpixVertsJit, x, y);
@@ -250,7 +243,7 @@ static double auth_lat(PJ *P, double alpha, int inverse) {
 
 
 /**
- * Return the HEAPJ_LPix projection of the longitude-latitude point lp on
+ * Return the HEALPix projection of the longitude-latitude point lp on
  * the unit sphere.
 **/
 static PJ_XY healpix_sphere(PJ_LP lp) {
@@ -353,10 +346,10 @@ static void dot_product(const double a[2][2], const double b[2], double *ret) {
 /**
  * Return the number of the polar cap, the pole point coordinates, and
  * the region that (x, y) lies in.
- * If inverse=0, then assume (x,y) lies in the image of the HEAPJ_LPix
+ * If inverse=0, then assume (x,y) lies in the image of the HEALPix
  * projection of the unit sphere.
  * If inverse=1, then assume (x,y) lies in the image of the
- * (north_square, south_square)-rHEAPJ_LPix projection of the unit sphere.
+ * (north_square, south_square)-rHEALPix projection of the unit sphere.
  **/
 static CapMap get_cap(double x, double y, int north_square, int south_square,
                       int inverse) {
@@ -411,8 +404,8 @@ static CapMap get_cap(double x, double y, int north_square, int south_square,
             capmap.cn = 0;
             return capmap;
         }
-        /* Polar Region, find the HEAPJ_LPix polar cap number that
-           x, y moves to when rHEAPJ_LPix polar square is disassembled. */
+        /* Polar Region, find the HEALPix polar cap number that
+           x, y moves to when rHEALPix polar square is disassembled. */
         if (capmap.region == CapMap::north) {
             if (y >= -x - M_FORTPI - EPS && y < x + 5*M_FORTPI - EPS) {
                 capmap.cn = (north_square + 1) % 4;
@@ -440,7 +433,7 @@ static CapMap get_cap(double x, double y, int north_square, int south_square,
 
 
 /**
- * Rearrange point (x, y) in the HEAPJ_LPix projection by
+ * Rearrange point (x, y) in the HEALPix projection by
  * combining the polar caps into two polar squares.
  * Put the north polar square in position north_square and
  * the south polar square in position south_square.
@@ -451,8 +444,6 @@ static CapMap get_cap(double x, double y, int north_square, int south_square,
 static PJ_XY combine_caps(double x, double y, int north_square, int south_square,
                        int inverse) {
     PJ_XY xy;
-    double v[2];
-    double c[2];
     double vector[2];
     double v_min_c[2];
     double ret_dot[2];
@@ -466,8 +457,8 @@ static PJ_XY combine_caps(double x, double y, int north_square, int south_square
         return xy;
     }
 
-    v[0] = x; v[1] = y;
-    c[0] = capmap.x; c[1] = capmap.y;
+    double v[] = {x, y};
+    double c[] = {capmap.x, capmap.y};
 
     if (inverse == 0) {
         /* Rotate (x, y) about its polar cap tip and then translate it to
@@ -497,11 +488,10 @@ static PJ_XY combine_caps(double x, double y, int north_square, int south_square
     vector_sub(v, c, v_min_c);
     dot_product(tmpRot, v_min_c, ret_dot);
     {
-        double a[2];
-        /* Workaround cppcheck git issue */
-        double* pa = a;
-        pa[0] = -3*M_FORTPI + ((inverse == 0) ? pole : capmap.cn) *M_HALFPI;
-        pa[1] = ((capmap.region == CapMap::north) ? 1 : -1) *M_HALFPI;
+        double a[] = {
+            -3*M_FORTPI + ((inverse == 0) ? pole : capmap.cn) *M_HALFPI,
+            ((capmap.region == CapMap::north) ? 1 : -1) *M_HALFPI
+        };
         vector_add(ret_dot, a, vector);
     }
 
@@ -513,23 +503,28 @@ static PJ_XY combine_caps(double x, double y, int north_square, int south_square
 
 static PJ_XY s_healpix_forward(PJ_LP lp, PJ *P) { /* sphere  */
     (void) P;
-    return healpix_sphere(lp);
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    return rotate(healpix_sphere(lp), -Q->rot_xy);
 }
 
 
 static PJ_XY e_healpix_forward(PJ_LP lp, PJ *P) { /* ellipsoid  */
     lp.phi = auth_lat(P, lp.phi, 0);
-    return healpix_sphere(lp);
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    return rotate(healpix_sphere(lp), -Q->rot_xy);
 }
 
 
 static PJ_LP s_healpix_inverse(PJ_XY xy, PJ *P) { /* sphere */
-    /* Check whether (x, y) lies in the HEAPJ_LPix image */
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    xy = rotate(xy, Q->rot_xy);
+
+    /* Check whether (x, y) lies in the HEALPix image */
     if (in_image(xy.x, xy.y, 0, 0, 0) == 0) {
         PJ_LP lp;
         lp.lam = HUGE_VAL;
         lp.phi = HUGE_VAL;
-        pj_ctx_set_errno(P->ctx, PJD_ERR_INVALID_X_OR_Y);
+        proj_context_errno_set(P->ctx, PROJ_ERR_COORD_TRANSFM_OUTSIDE_PROJECTION_DOMAIN);
         return lp;
     }
     return healpix_spherhealpix_e_inverse(xy);
@@ -538,12 +533,14 @@ static PJ_LP s_healpix_inverse(PJ_XY xy, PJ *P) { /* sphere */
 
 static PJ_LP e_healpix_inverse(PJ_XY xy, PJ *P) { /* ellipsoid */
     PJ_LP lp = {0.0,0.0};
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    xy = rotate(xy, Q->rot_xy);
 
-    /* Check whether (x, y) lies in the HEAPJ_LPix image. */
+    /* Check whether (x, y) lies in the HEALPix image. */
     if (in_image(xy.x, xy.y, 0, 0, 0) == 0) {
         lp.lam = HUGE_VAL;
         lp.phi = HUGE_VAL;
-        pj_ctx_set_errno(P->ctx, PJD_ERR_INVALID_X_OR_Y);
+        proj_context_errno_set(P->ctx, PROJ_ERR_COORD_TRANSFM_OUTSIDE_PROJECTION_DOMAIN);
         return lp;
     }
     lp = healpix_spherhealpix_e_inverse(xy);
@@ -572,12 +569,12 @@ static PJ_XY e_rhealpix_forward(PJ_LP lp, PJ *P) { /* ellipsoid */
 static PJ_LP s_rhealpix_inverse(PJ_XY xy, PJ *P) { /* sphere */
     struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
 
-    /* Check whether (x, y) lies in the rHEAPJ_LPix image. */
+    /* Check whether (x, y) lies in the rHEALPix image. */
     if (in_image(xy.x, xy.y, 1, Q->north_square, Q->south_square) == 0) {
         PJ_LP lp;
         lp.lam = HUGE_VAL;
         lp.phi = HUGE_VAL;
-        pj_ctx_set_errno(P->ctx, PJD_ERR_INVALID_X_OR_Y);
+        proj_context_errno_set(P->ctx, PROJ_ERR_COORD_TRANSFM_OUTSIDE_PROJECTION_DOMAIN);
         return lp;
     }
     xy = combine_caps(xy.x, xy.y, Q->north_square, Q->south_square, 1);
@@ -589,11 +586,11 @@ static PJ_LP e_rhealpix_inverse(PJ_XY xy, PJ *P) { /* ellipsoid */
     struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
     PJ_LP lp = {0.0,0.0};
 
-    /* Check whether (x, y) lies in the rHEAPJ_LPix image. */
+    /* Check whether (x, y) lies in the rHEALPix image. */
     if (in_image(xy.x, xy.y, 1, Q->north_square, Q->south_square) == 0) {
         lp.lam = HUGE_VAL;
         lp.phi = HUGE_VAL;
-        pj_ctx_set_errno(P->ctx, PJD_ERR_INVALID_X_OR_Y);
+        proj_context_errno_set(P->ctx, PROJ_ERR_COORD_TRANSFM_OUTSIDE_PROJECTION_DOMAIN);
         return lp;
     }
     xy = combine_caps(xy.x, xy.y, Q->north_square, Q->south_square, 1);
@@ -610,22 +607,25 @@ static PJ *destructor (PJ *P, int errlev) {                        /* Destructor
     if (nullptr==P->opaque)
         return pj_default_destructor (P, errlev);
 
-    pj_dealloc (static_cast<struct pj_opaque*>(P->opaque)->apa);
+    free (static_cast<struct pj_opaque*>(P->opaque)->apa);
     return pj_default_destructor (P, errlev);
 }
 
 
 PJ *PROJECTION(healpix) {
-    struct pj_opaque *Q = static_cast<struct pj_opaque*>(pj_calloc (1, sizeof (struct pj_opaque)));
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(calloc (1, sizeof (struct pj_opaque)));
     if (nullptr==Q)
-        return pj_default_destructor (P, ENOMEM);
+        return pj_default_destructor (P, PROJ_ERR_OTHER /*ENOMEM*/);
     P->opaque = Q;
     P->destructor = destructor;
 
+    double angle = pj_param(P->ctx, P->params,"drot_xy").f;
+    Q->rot_xy = PJ_TORAD(angle);
+
     if (P->es != 0.0) {
         Q->apa = pj_authset(P->es);             /* For auth_lat(). */
         if (nullptr==Q->apa)
-            return destructor(P, ENOMEM);
+            return destructor(P, PROJ_ERR_OTHER /*ENOMEM*/);
         Q->qp = pj_qsfn(1.0, P->e, P->one_es);  /* For auth_lat(). */
         P->a = P->a*sqrt(0.5*Q->qp);            /* Set P->a to authalic radius. */
         pj_calc_ellipsoid_params (P, P->a, P->es);  /* Ensure we have a consistent parameter set */
@@ -641,9 +641,9 @@ PJ *PROJECTION(healpix) {
 
 
 PJ *PROJECTION(rhealpix) {
-    struct pj_opaque *Q = static_cast<struct pj_opaque*>(pj_calloc (1, sizeof (struct pj_opaque)));
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(calloc (1, sizeof (struct pj_opaque)));
     if (nullptr==Q)
-        return pj_default_destructor (P, ENOMEM);
+        return pj_default_destructor (P, PROJ_ERR_OTHER /*ENOMEM*/);
     P->opaque = Q;
     P->destructor = destructor;
 
@@ -652,13 +652,19 @@ PJ *PROJECTION(rhealpix) {
 
     /* Check for valid north_square and south_square inputs. */
     if (Q->north_square < 0 || Q->north_square > 3)
-        return destructor (P, PJD_ERR_AXIS);
+    {
+        proj_log_error(P, _("Invalid value for north_square: it should be in [0,3] range."));
+        return destructor(P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+    }
     if (Q->south_square < 0 || Q->south_square > 3)
-        return destructor (P, PJD_ERR_AXIS);
+    {
+        proj_log_error(P, _("Invalid value for south_square: it should be in [0,3] range."));
+        return destructor(P, PROJ_ERR_INVALID_OP_ILLEGAL_ARG_VALUE);
+    }
     if (P->es != 0.0) {
         Q->apa = pj_authset(P->es); /* For auth_lat(). */
         if (nullptr==Q->apa)
-            return destructor(P, ENOMEM);
+            return destructor(P, PROJ_ERR_OTHER /*ENOMEM*/);
         Q->qp = pj_qsfn(1.0, P->e, P->one_es); /* For auth_lat(). */
         P->a = P->a*sqrt(0.5*Q->qp); /* Set P->a to authalic radius. */
         P->ra = 1.0/P->a;
