@@ -46,9 +46,8 @@ TEST(gie, cart_selftest) {
     PJ_CONTEXT *ctx;
     PJ *P;
     PJ_COORD a, b, obs[2];
-    PJ_COORD coord[2];
+    PJ_COORD coord[3];
 
-    int err;
     size_t n, sz;
     double dist, h, t;
     const char *const args[3] = {"proj=utm", "zone=32", "ellps=GRS80"};
@@ -89,16 +88,6 @@ TEST(gie, cart_selftest) {
     /* Clear any previous error */
     proj_errno_reset(P);
 
-    /* Invalid projection */
-    a = proj_trans(P, static_cast<PJ_DIRECTION>(42), a);
-    ASSERT_EQ(a.lpz.lam, HUGE_VAL);
-
-    err = proj_errno(P);
-    ASSERT_NE(err, 0);
-
-    /* Clear error again */
-    proj_errno_reset(P);
-
     /* Clean up */
     proj_destroy(P);
 
@@ -211,7 +200,7 @@ TEST(gie, cart_selftest) {
 
     coord[0] = proj_coord(proj_torad(12), proj_torad(55), 45, 0);
     coord[1] = proj_coord(proj_torad(12), proj_torad(56), 50, 0);
-    ASSERT_FALSE(proj_trans_array(P, PJ_FWD, 2, coord));
+    ASSERT_EQ(proj_trans_array(P, PJ_FWD, 2, coord), 0);
 
     ASSERT_EQ(a.lpz.lam, coord[0].lpz.lam);
     ASSERT_EQ(a.lpz.phi, coord[0].lpz.phi);
@@ -220,6 +209,35 @@ TEST(gie, cart_selftest) {
     ASSERT_EQ(b.lpz.phi, coord[1].lpz.phi);
     ASSERT_EQ(b.lpz.z, coord[1].lpz.z);
 
+    /* test proj_trans_array () with two failed points for the same reason */
+
+    coord[0] =
+        proj_coord(proj_torad(12), proj_torad(95), 45, 0); // invalid latitude
+    coord[1] = proj_coord(proj_torad(12), proj_torad(56), 50, 0);
+    coord[2] =
+        proj_coord(proj_torad(12), proj_torad(95), 45, 0); // invalid latitude
+    ASSERT_EQ(proj_trans_array(P, PJ_FWD, 3, coord),
+              PROJ_ERR_COORD_TRANSFM_INVALID_COORD);
+
+    ASSERT_EQ(HUGE_VAL, coord[0].lpz.lam);
+    ASSERT_EQ(HUGE_VAL, coord[0].lpz.phi);
+    ASSERT_EQ(HUGE_VAL, coord[0].lpz.z);
+    ASSERT_EQ(b.lpz.lam, coord[1].lpz.lam);
+    ASSERT_EQ(b.lpz.phi, coord[1].lpz.phi);
+    ASSERT_EQ(b.lpz.z, coord[1].lpz.z);
+    ASSERT_EQ(HUGE_VAL, coord[2].lpz.lam);
+    ASSERT_EQ(HUGE_VAL, coord[2].lpz.phi);
+    ASSERT_EQ(HUGE_VAL, coord[2].lpz.z);
+
+    /* test proj_trans_array () with two failed points for different reasons */
+
+    coord[0] =
+        proj_coord(proj_torad(12), proj_torad(95), 45, 0); // invalid latitude
+    coord[1] =
+        proj_coord(proj_torad(105), proj_torad(0), 45,
+                   0); // in the equatorial axis, at 90° of the central meridian
+    ASSERT_EQ(proj_trans_array(P, PJ_FWD, 2, coord), PROJ_ERR_COORD_TRANSFM);
+
     /* Clean up  after proj_trans_* tests */
     proj_destroy(P);
 }
@@ -334,7 +352,6 @@ TEST(gie, info_functions) {
 
     const PJ_OPERATIONS *oper_list;
     const PJ_ELLPS *ellps_list;
-    const PJ_UNITS *unit_list;
     const PJ_PRIME_MERIDIANS *pm_list;
 
     char buf[40];
@@ -348,7 +365,9 @@ TEST(gie, info_functions) {
 
     /* proj_info()                                                            */
     /* this one is difficult to test, since the output changes with the setup */
+    putenv(const_cast<char *>("PROJ_SKIP_READ_USER_WRITABLE_DIRECTORY="));
     info = proj_info();
+    putenv(const_cast<char *>("PROJ_SKIP_READ_USER_WRITABLE_DIRECTORY=YES"));
 
     if (info.version[0] != '\0') {
         char tmpstr[64];
@@ -360,6 +379,9 @@ TEST(gie, info_functions) {
         ASSERT_NE(std::string(info.searchpath), std::string());
     }
 
+    ASSERT_TRUE(std::string(info.searchpath).find("/proj") !=
+                std::string::npos);
+
     /* proj_pj_info() */
     {
         P = proj_create(PJ_DEFAULT_CTX,
@@ -379,13 +401,18 @@ TEST(gie, info_functions) {
     proj_destroy(P);
 
     /* proj_grid_info() */
-    grid_info = proj_grid_info("null");
+    grid_info = proj_grid_info("tests/test_hgrid.tif");
     ASSERT_NE(std::string(grid_info.filename), "");
-    ASSERT_EQ(std::string(grid_info.gridname), "null");
+    ASSERT_EQ(std::string(grid_info.gridname), "tests/test_hgrid.tif");
+    ASSERT_EQ(std::string(grid_info.format), "gtiff");
 
     grid_info = proj_grid_info("nonexistinggrid");
     ASSERT_EQ(std::string(grid_info.filename), "");
 
+    // File exists, but is not a grid
+    grid_info = proj_grid_info("proj.db");
+    ASSERT_EQ(std::string(grid_info.filename), "");
+
     /* proj_init_info() */
     init_info = proj_init_info("unknowninit");
     ASSERT_EQ(std::string(init_info.filename), "");
@@ -442,11 +469,6 @@ TEST(gie, info_functions) {
         n++;
     ASSERT_NE(n, 0U);
 
-    n = 0;
-    for (unit_list = proj_list_units(); unit_list->id; ++unit_list)
-        n++;
-    ASSERT_NE(n, 0U);
-
     n = 0;
     for (pm_list = proj_list_prime_meridians(); pm_list->id; ++pm_list)
         n++;
@@ -544,7 +566,7 @@ static void test_time(const char *args, double tol, double t_in, double t_exp) {
     out = proj_trans(P, PJ_INV, out);
     EXPECT_NEAR(out.xyzt.t, t_in, tol);
 
-    pj_free(P);
+    proj_destroy(P);
 
     proj_log_level(NULL, PJ_LOG_NONE);
 }
